@@ -2,7 +2,7 @@
    Input line filename/username/hostname/variable/command completion.
    (Let mc type for you...)
 
-   Copyright (C) 1995-2019
+   Copyright (C) 1995-2022
    Free Software Foundation, Inc.
 
    Written by:
@@ -53,8 +53,6 @@
 #include "lib/util.h"
 #include "lib/widget.h"
 
-#include "input_complete.h"
-
 /*** global variables ****************************************************************************/
 
 /* Linux declares environ in <unistd.h>, so don't repeat it here. */
@@ -95,7 +93,7 @@ static char **hosts = NULL;
 static char **hosts_p = NULL;
 static int hosts_alloclen = 0;
 
-static int query_height, query_width;
+static int complete_height, complete_width;
 static WInput *input;
 static int min_end;
 static int start = 0;
@@ -141,7 +139,7 @@ filename_completion_function (const char *text, int state, input_complete_t flag
     static vfs_path_t *dirname_vpath = NULL;
 
     gboolean isdir = TRUE, isexec = FALSE;
-    struct dirent *entry = NULL;
+    struct vfs_dirent *entry = NULL;
 
     SHOW_C_CTX ("filename_completion_function");
 
@@ -170,7 +168,7 @@ filename_completion_function (const char *text, int state, input_complete_t flag
         g_free (dirname);
         g_free (filename);
         g_free (users_dirname);
-        vfs_path_free (dirname_vpath);
+        vfs_path_free (dirname_vpath, TRUE);
 
         if ((*text != '\0') && (temp = strrchr (text, PATH_SEP)) != NULL)
         {
@@ -257,7 +255,7 @@ filename_completion_function (const char *text, int state, input_complete_t flag
                 /* stat failed, strange. not a dir in any case */
                 isdir = FALSE;
             }
-            vfs_path_free (tmp_vpath);
+            vfs_path_free (tmp_vpath, TRUE);
         }
 
         if ((flags & INPUT_COMPLETE_COMMANDS) != 0 && (isexec || isdir))
@@ -276,7 +274,7 @@ filename_completion_function (const char *text, int state, input_complete_t flag
             directory = NULL;
         }
         MC_PTR_FREE (dirname);
-        vfs_path_free (dirname_vpath);
+        vfs_path_free (dirname_vpath, TRUE);
         dirname_vpath = NULL;
         MC_PTR_FREE (filename);
         MC_PTR_FREE (users_dirname);
@@ -985,7 +983,11 @@ insert_text (WInput * in, char *text, ssize_t size)
 
     text_len = strlen (text);
     buff_len = str_length (in->buffer);
-    size = MIN (size, (ssize_t) text_len) + start - end;
+    if (size < 0)
+        size = (ssize_t) text_len;
+    else
+        size = MIN (size, (ssize_t) text_len);
+    size += start - end;
     if (strlen (in->buffer) + size >= (size_t) in->current_max_size)
     {
         /* Expand the buffer */
@@ -1015,10 +1017,11 @@ insert_text (WInput * in, char *text, ssize_t size)
 /* --------------------------------------------------------------------------------------------- */
 
 static cb_ret_t
-query_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
+complete_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     static int bl = 0;
 
+    WGroup *g = GROUP (w);
     WDialog *h = DIALOG (w);
 
     switch (msg)
@@ -1057,17 +1060,17 @@ query_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
 
                 new_end = str_get_prev_char (&input->buffer[end]) - input->buffer;
 
-                for (i = 0, e = listbox_get_first_link (LISTBOX (h->current->data));
+                for (i = 0, e = listbox_get_first_link (LISTBOX (g->current->data));
                      e != NULL; i++, e = g_list_next (e))
                 {
                     WLEntry *le = LENTRY (e->data);
 
                     if (strncmp (input->buffer + start, le->text, new_end - start) == 0)
                     {
-                        listbox_select_entry (LISTBOX (h->current->data), i);
+                        listbox_select_entry (LISTBOX (g->current->data), i);
                         end = new_end;
                         input_handle_char (input, parm);
-                        widget_draw (WIDGET (h->current->data));
+                        widget_draw (WIDGET (g->current->data));
                         break;
                     }
                 }
@@ -1078,7 +1081,7 @@ query_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
             if (parm < 32 || parm > 255)
             {
                 bl = 0;
-                if (input_key_is_in_map (input, parm) != 2)
+                if (widget_lookup_key (WIDGET (input), parm) != CK_Complete)
                     return MSG_NOT_HANDLED;
 
                 if (end == min_end)
@@ -1111,7 +1114,7 @@ query_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
                     break;
                 }
 
-                for (i = 0, e = listbox_get_first_link (LISTBOX (h->current->data));
+                for (i = 0, e = listbox_get_first_link (LISTBOX (g->current->data));
                      e != NULL; i++, e = g_list_next (e))
                 {
                     WLEntry *le = LENTRY (e->data);
@@ -1122,7 +1125,7 @@ query_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
                         if (need_redraw == 0)
                         {
                             need_redraw = 1;
-                            listbox_select_entry (LISTBOX (h->current->data), i);
+                            listbox_select_entry (LISTBOX (g->current->data), i);
                             last_text = le->text;
                         }
                         else
@@ -1173,7 +1176,7 @@ query_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
                 if (need_redraw == 2)
                 {
                     insert_text (input, last_text, low);
-                    widget_draw (WIDGET (h->current->data));
+                    widget_draw (WIDGET (g->current->data));
                 }
                 else if (need_redraw == 1)
                 {
@@ -1197,7 +1200,7 @@ static gboolean
 complete_engine (WInput * in, int what_to_do)
 {
     if (in->completions != NULL && str_offset_to_pos (in->buffer, in->point) != end)
-        input_free_completions (in);
+        input_complete_free (in);
 
     if (in->completions == NULL)
         complete_engine_fill_completions (in);
@@ -1211,10 +1214,10 @@ complete_engine (WInput * in, int what_to_do)
         {
             char *lc_complete = in->completions[0];
 
-            if (!insert_text (in, lc_complete, strlen (lc_complete)) || in->completions[1] != NULL)
+            if (!insert_text (in, lc_complete, -1) || in->completions[1] != NULL)
                 tty_beep ();
             else
-                input_free_completions (in);
+                input_complete_free (in);
         }
 
         if ((what_to_do & DO_QUERY) != 0 && in->completions != NULL && in->completions[1] != NULL)
@@ -1223,8 +1226,8 @@ complete_engine (WInput * in, int what_to_do)
             int x, y, w, h;
             int start_x, start_y;
             char **p, *q;
-            WDialog *query_dlg;
-            WListbox *query_list;
+            WDialog *complete_dlg;
+            WListbox *complete_list;
 
             for (p = in->completions + 1; *p != NULL; count++, p++)
             {
@@ -1261,28 +1264,29 @@ complete_engine (WInput * in, int what_to_do)
 
             input = in;
             min_end = end;
-            query_height = h;
-            query_width = w;
+            complete_height = h;
+            complete_width = w;
 
-            query_dlg = dlg_create (TRUE, y, x, query_height, query_width, WPOS_KEEP_DEFAULT, TRUE,
-                                    dialog_colors, query_callback, NULL, "[Completion]", NULL);
-            query_list = listbox_new (1, 1, h - 2, w - 2, FALSE, NULL);
-            add_widget (query_dlg, query_list);
+            complete_dlg =
+                dlg_create (TRUE, y, x, complete_height, complete_width, WPOS_KEEP_DEFAULT, TRUE,
+                            dialog_colors, complete_callback, NULL, "[Completion]", NULL);
+            complete_list = listbox_new (1, 1, h - 2, w - 2, FALSE, NULL);
+            group_add_widget (GROUP (complete_dlg), complete_list);
 
             for (p = in->completions + 1; *p != NULL; p++)
-                listbox_add_item (query_list, LISTBOX_APPEND_AT_END, 0, *p, NULL, FALSE);
+                listbox_add_item (complete_list, LISTBOX_APPEND_AT_END, 0, *p, NULL, FALSE);
 
-            i = dlg_run (query_dlg);
+            i = dlg_run (complete_dlg);
             q = NULL;
             if (i == B_ENTER)
             {
-                listbox_get_current (query_list, &q, NULL);
+                listbox_get_current (complete_list, &q, NULL);
                 if (q != NULL)
-                    insert_text (in, q, strlen (q));
+                    insert_text (in, q, -1);
             }
             if (q != NULL || end != min_end)
-                input_free_completions (in);
-            dlg_destroy (query_dlg);
+                input_complete_free (in);
+            widget_destroy (WIDGET (complete_dlg));
 
             /* B_USER if user wants to start over again */
             return (i == B_USER);
@@ -1436,7 +1440,7 @@ complete_engine_fill_completions (WInput * in)
 
 /* declared in lib/widget/input.h */
 void
-complete (WInput * in)
+input_complete (WInput * in)
 {
     int engine_flags;
 
@@ -1458,3 +1462,12 @@ complete (WInput * in)
 }
 
 /* --------------------------------------------------------------------------------------------- */
+
+void
+input_complete_free (WInput * in)
+{
+    g_strfreev (in->completions);
+    in->completions = NULL;
+}
+
+/* --------------------------------------------------------------------------------------------- */
