@@ -1,7 +1,7 @@
 /*
    File difference viewer
 
-   Copyright (C) 2007-2019
+   Copyright (C) 2007-2022
    Free Software Foundation, Inc.
 
    Written by:
@@ -28,8 +28,10 @@
 
 
 #include <config.h>
+
 #include <ctype.h>
 #include <errno.h>
+#include <stddef.h>             /* ptrdiff_t */
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -55,7 +57,7 @@
 #include "src/filemanager/layout.h"     /* Needed for get_current_index and get_other_panel */
 
 #include "src/execute.h"        /* toggle_subshell() */
-#include "src/keybind-defaults.h"
+#include "src/keymap.h"
 #include "src/setup.h"
 #include "src/history.h"
 #ifdef HAVE_CHARSET
@@ -176,8 +178,8 @@ open_temp (void **name)
                  _("Cannot create temporary diff file\n%s"), unix_error_string (errno));
         return -1;
     }
-    *name = g_strdup (vfs_path_as_str (diff_file_name));
-    vfs_path_free (diff_file_name);
+
+    *name = vfs_path_free (diff_file_name, FALSE);
     return fd;
 }
 
@@ -2250,7 +2252,7 @@ do_merge_hunk (WDiff * dview, action_direction_t merge_direction)
             (void) res;
         }
         mc_unlink (merge_file_name_vpath);
-        vfs_path_free (merge_file_name_vpath);
+        vfs_path_free (merge_file_name_vpath, TRUE);
     }
 }
 
@@ -2774,7 +2776,7 @@ dview_status (const WDiff * dview, diff_place_t ord, int width, int c)
 
     vpath = vfs_path_from_str (dview->label[ord]);
     path = vfs_path_to_str_flags (vpath, 0, VPF_STRIP_HOME | VPF_STRIP_PASSWORD);
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
     buf = str_term_trim (path, filename_width);
     if (ord == DIFF_LEFT)
         tty_printf ("%s%-*s %6d+%-4d Col %-4d ", dview->merged[ord] ? "* " : "  ", filename_width,
@@ -2882,7 +2884,7 @@ dview_update (WDiff * dview)
 static void
 dview_edit (WDiff * dview, diff_place_t ord)
 {
-    WDialog *h;
+    Widget *h;
     gboolean h_modal;
     int linenum, lineofs;
 
@@ -2892,23 +2894,23 @@ dview_edit (WDiff * dview, diff_place_t ord)
         return;
     }
 
-    h = WIDGET (dview)->owner;
-    h_modal = widget_get_state (WIDGET (h), WST_MODAL);
+    h = WIDGET (WIDGET (dview)->owner);
+    h_modal = widget_get_state (h, WST_MODAL);
 
     get_line_numbers (dview->a[ord], dview->skip_rows, &linenum, &lineofs);
 
     /* disallow edit file in several editors */
-    widget_set_state (WIDGET (h), WST_MODAL, TRUE);
+    widget_set_state (h, WST_MODAL, TRUE);
 
     {
         vfs_path_t *tmp_vpath;
 
         tmp_vpath = vfs_path_from_str (dview->file[ord]);
         edit_file_at_line (tmp_vpath, use_internal_edit, linenum);
-        vfs_path_free (tmp_vpath);
+        vfs_path_free (tmp_vpath, TRUE);
     }
 
-    widget_set_state (WIDGET (h), WST_MODAL, h_modal);
+    widget_set_state (h, WST_MODAL, h_modal);
     dview_redo (dview);
     dview_update (dview);
 }
@@ -2965,21 +2967,18 @@ dview_goto_cmd (WDiff * dview, diff_place_t ord)
 static void
 dview_labels (WDiff * dview)
 {
-    Widget *d;
-    WDialog *h;
+    Widget *d = WIDGET (dview);
     WButtonBar *b;
 
-    d = WIDGET (dview);
-    h = d->owner;
-    b = find_buttonbar (h);
+    b = find_buttonbar (DIALOG (d->owner));
 
-    buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), diff_map, d);
-    buttonbar_set_label (b, 2, Q_ ("ButtonBar|Save"), diff_map, d);
-    buttonbar_set_label (b, 4, Q_ ("ButtonBar|Edit"), diff_map, d);
-    buttonbar_set_label (b, 5, Q_ ("ButtonBar|Merge"), diff_map, d);
-    buttonbar_set_label (b, 7, Q_ ("ButtonBar|Search"), diff_map, d);
-    buttonbar_set_label (b, 9, Q_ ("ButtonBar|Options"), diff_map, d);
-    buttonbar_set_label (b, 10, Q_ ("ButtonBar|Quit"), diff_map, d);
+    buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), d->keymap, d);
+    buttonbar_set_label (b, 2, Q_ ("ButtonBar|Save"), d->keymap, d);
+    buttonbar_set_label (b, 4, Q_ ("ButtonBar|Edit"), d->keymap, d);
+    buttonbar_set_label (b, 5, Q_ ("ButtonBar|Merge"), d->keymap, d);
+    buttonbar_set_label (b, 7, Q_ ("ButtonBar|Search"), d->keymap, d);
+    buttonbar_set_label (b, 9, Q_ ("ButtonBar|Options"), d->keymap, d);
+    buttonbar_set_label (b, 10, Q_ ("ButtonBar|Quit"), d->keymap, d);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -3292,12 +3291,11 @@ dview_handle_key (WDiff * dview, int key)
     key = convert_from_input_c (key);
 #endif
 
-    command = keybind_lookup_keymap_command (diff_map, key);
-    if ((command != CK_IgnoreKey) && (dview_execute_cmd (dview, command) == MSG_HANDLED))
-        return MSG_HANDLED;
+    command = widget_lookup_key (WIDGET (dview), key);
+    if (command == CK_IgnoreKey)
+        return MSG_NOT_HANDLED;
 
-    /* Key not used */
-    return MSG_NOT_HANDLED;
+    return dview_execute_cmd (dview, command);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -3306,7 +3304,7 @@ static cb_ret_t
 dview_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
     WDiff *dview = (WDiff *) w;
-    WDialog *h = w->owner;
+    WDialog *h = DIALOG (w->owner);
     cb_ret_t i;
 
     switch (msg)
@@ -3339,6 +3337,7 @@ dview_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
         return i;
 
     case MSG_RESIZE:
+        widget_default_callback (w, NULL, MSG_RESIZE, 0, data);
         dview_compute_areas (dview);
         return MSG_HANDLED;
 
@@ -3397,7 +3396,7 @@ dview_dialog_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm,
         return dview_execute_cmd (NULL, parm);
 
     case MSG_VALIDATE:
-        dview = (WDiff *) find_widget_type (h, dview_callback);
+        dview = (WDiff *) widget_find_by_type (CONST_WIDGET (h), dview_callback);
         /* don't stop the dialog before final decision */
         widget_set_state (w, WST_ACTIVE, TRUE);
         if (dview_ok_to_exit (dview))
@@ -3420,7 +3419,7 @@ dview_get_title (const WDialog * h, size_t len)
     size_t len1;
     GString *title;
 
-    dview = (const WDiff *) find_widget_type (h, dview_callback);
+    dview = (const WDiff *) widget_find_by_type (CONST_WIDGET (h), dview_callback);
     len1 = (len - str_term_width1 (_("Diff:")) - strlen (modified) - 3) / 2;
 
     title = g_string_sized_new (len);
@@ -3444,6 +3443,7 @@ diff_view (const char *file1, const char *file2, const char *label1, const char
     Widget *w;
     WDialog *dview_dlg;
     Widget *dw;
+    WGroup *g;
 
     /* Create dialog and widgets, put them on the dialog */
     dview_dlg =
@@ -3452,14 +3452,17 @@ diff_view (const char *file1, const char *file2, const char *label1, const char
     dw = WIDGET (dview_dlg);
     widget_want_tab (dw, TRUE);
 
+    g = GROUP (dview_dlg);
+
     dview = g_new0 (WDiff, 1);
     w = WIDGET (dview);
     widget_init (w, dw->y, dw->x, dw->lines - 1, dw->cols, dview_callback, dview_mouse_callback);
     w->options |= WOP_SELECTABLE;
-    add_widget_autopos (dview_dlg, w, WPOS_KEEP_ALL, NULL);
+    w->keymap = diff_map;
+    group_add_widget_autopos (g, w, WPOS_KEEP_ALL, NULL);
 
-    w = WIDGET (buttonbar_new (TRUE));
-    add_widget_autopos (dview_dlg, w, w->pos_flags, NULL);
+    w = WIDGET (buttonbar_new ());
+    group_add_widget_autopos (g, w, w->pos_flags, NULL);
 
     dview_dlg->get_title = dview_get_title;
 
@@ -3469,7 +3472,7 @@ diff_view (const char *file1, const char *file2, const char *label1, const char
         dlg_run (dview_dlg);
 
     if (error != 0 || widget_get_state (dw, WST_CLOSED))
-        dlg_destroy (dview_dlg);
+        widget_destroy (dw);
 
     return error == 0 ? 1 : 0;
 }
@@ -3509,7 +3512,7 @@ do \
                 changed = (mtime != st##n.st_mtime); \
         } \
         mc_ungetlocalcopy (file##n, real_file##n, changed); \
-        vfs_path_free (real_file##n); \
+        vfs_path_free (real_file##n, TRUE); \
     } \
 } \
 while (0)
@@ -3532,22 +3535,24 @@ dview_diff_cmd (const void *f0, const void *f1)
             const WPanel *panel1 = (const WPanel *) f1;
 
             file0 =
-                vfs_path_append_new (panel0->cwd_vpath, selection (panel0)->fname, (char *) NULL);
+                vfs_path_append_new (panel0->cwd_vpath, selection (panel0)->fname->str,
+                                     (char *) NULL);
             is_dir0 = S_ISDIR (selection (panel0)->st.st_mode);
             if (is_dir0)
             {
                 message (D_ERROR, MSG_ERROR, _("\"%s\" is a directory"),
-                         path_trunc (selection (panel0)->fname, 30));
+                         path_trunc (selection (panel0)->fname->str, 30));
                 goto ret;
             }
 
             file1 =
-                vfs_path_append_new (panel1->cwd_vpath, selection (panel1)->fname, (char *) NULL);
+                vfs_path_append_new (panel1->cwd_vpath, selection (panel1)->fname->str,
+                                     (char *) NULL);
             is_dir1 = S_ISDIR (selection (panel1)->st.st_mode);
             if (is_dir1)
             {
                 message (D_ERROR, MSG_ERROR, _("\"%s\" is a directory"),
-                         path_trunc (selection (panel1)->fname, 30));
+                         path_trunc (selection (panel1)->fname->str, 30));
                 goto ret;
             }
             break;
@@ -3630,8 +3635,8 @@ dview_diff_cmd (const void *f0, const void *f1)
         message (D_ERROR, MSG_ERROR, _("Two files are needed to compare"));
 
   ret:
-    vfs_path_free (file1);
-    vfs_path_free (file0);
+    vfs_path_free (file1, TRUE);
+    vfs_path_free (file0, TRUE);
 
     return (rv != 0);
 }
