@@ -2,7 +2,7 @@
    Routines invoked by a function key
    They normally operate on the current panel.
 
-   Copyright (C) 1994-2021
+   Copyright (C) 1994-2022
    Free Software Foundation, Inc.
 
    Written by:
@@ -80,10 +80,8 @@
 #endif
 
 #include "fileopctx.h"
-#include "file.h"               /* file operation routines */
 #include "filenot.h"
 #include "hotlist.h"            /* hotlist_show() */
-#include "panel.h"              /* WPanel */
 #include "tree.h"               /* tree_chdir() */
 #include "filemanager.h"        /* change_panel() */
 #include "command.h"            /* cmdline */
@@ -769,66 +767,6 @@ edit_cmd_new (void)
     vfs_path_free (fname_vpath, TRUE);
 }
 
-/* --------------------------------------------------------------------------------------------- */
-/** Invoked by F5.  Copy, default to the other panel.  */
-
-void
-copy_cmd (WPanel * panel)
-{
-    save_cwds_stat ();
-
-    if (panel_operate (panel, OP_COPY, FALSE))
-    {
-        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-        repaint_screen ();
-    }
-}
-
-/* --------------------------------------------------------------------------------------------- */
-/** Invoked by F6.  Move/rename, default to the other panel, ignore marks.  */
-
-void
-rename_cmd (WPanel * panel)
-{
-    save_cwds_stat ();
-
-    if (panel_operate (panel, OP_MOVE, FALSE))
-    {
-        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-        repaint_screen ();
-    }
-}
-
-/* --------------------------------------------------------------------------------------------- */
-/** Invoked by F15.  Copy, default to the same panel, ignore marks.  */
-
-void
-copy_cmd_local (WPanel * panel)
-{
-    save_cwds_stat ();
-
-    if (panel_operate (panel, OP_COPY, TRUE))
-    {
-        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-        repaint_screen ();
-    }
-}
-
-/* --------------------------------------------------------------------------------------------- */
-/** Invoked by F16.  Move/rename, default to the same panel.  */
-
-void
-rename_cmd_local (WPanel * panel)
-{
-    save_cwds_stat ();
-
-    if (panel_operate (panel, OP_MOVE, TRUE))
-    {
-        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-        repaint_screen ();
-    }
-}
-
 /* --------------------------------------------------------------------------------------------- */
 
 void
@@ -880,35 +818,6 @@ mkdir_cmd (WPanel * panel)
     g_free (dir);
 }
 
-/* --------------------------------------------------------------------------------------------- */
-
-void
-delete_cmd (WPanel * panel)
-{
-    save_cwds_stat ();
-
-    if (panel_operate (panel, OP_DELETE, FALSE))
-    {
-        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-        repaint_screen ();
-    }
-}
-
-/* --------------------------------------------------------------------------------------------- */
-/** Invoked by F18.  Remove selected file, regardless of marked files.  */
-
-void
-delete_cmd_local (WPanel * panel)
-{
-    save_cwds_stat ();
-
-    if (panel_operate (panel, OP_DELETE, TRUE))
-    {
-        update_panels (UP_OPTIMIZE, UP_KEEPSEL);
-        repaint_screen ();
-    }
-}
-
 /* --------------------------------------------------------------------------------------------- */
 /** Invoked from the left/right menus */
 
@@ -1329,17 +1238,6 @@ fishlink_cmd (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-#ifdef ENABLE_VFS_SMB
-void
-smblink_cmd (void)
-{
-    nice_cd (_("SMB link to machine"), _(machine_str),
-             "[SMB File System]", ":smblink_cmd: SMB link to machine ", "smb://", 0, TRUE);
-}
-#endif /* ENABLE_VFS_SMB */
-
-/* --------------------------------------------------------------------------------------------- */
-
 #ifdef ENABLE_VFS_UNDELFS
 void
 undelete_cmd (void)
