@@ -16,6 +16,16 @@
 #define DIR_LIST_MIN_SIZE 128
 #define DIR_LIST_RESIZE_STEP 128
 
+typedef enum
+{
+    DIR_OPEN = 0,
+    DIR_READ,
+    DIR_CLOSE
+} dir_list_cb_state_t;
+
+/* dir_list callback */
+typedef void (*dir_list_cb_fn) (dir_list_cb_state_t state, void *data);
+
 /*** enums ***************************************************************************************/
 
 /*** structures declarations (and typedefs of structures)*****************************************/
@@ -28,6 +38,7 @@ typedef struct
     file_entry_t *list; /**< list of file_entry_t objects */
     int size;           /**< number of allocated elements in list (capacity) */
     int len;            /**< number of used elements in list */
+    dir_list_cb_fn callback;    /**< callback to visualize of directory read */
 } dir_list;
 
 /**
@@ -48,15 +59,16 @@ gboolean dir_list_grow (dir_list * list, int delta);
 gboolean dir_list_append (dir_list * list, const char *fname, const struct stat *st,
                           gboolean link_to_dir, gboolean stale_link);
 
-void dir_list_load (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
-                    const dir_sort_options_t * sort_op, const char *fltr);
-void dir_list_reload (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
-                      const dir_sort_options_t * sort_op, const char *fltr);
+gboolean dir_list_load (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
+                        const dir_sort_options_t * sort_op, const char *filter);
+gboolean dir_list_reload (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
+                          const dir_sort_options_t * sort_op, const char *filter);
 void dir_list_sort (dir_list * list, GCompareFunc sort, const dir_sort_options_t * sort_op);
 gboolean dir_list_init (dir_list * list);
 void dir_list_clean (dir_list * list);
 void dir_list_free_list (dir_list * list);
-gboolean handle_path (const char *path, struct stat *buf1, int *link_to_dir, int *stale_link);
+gboolean handle_path (const char *path, struct stat *buf1, gboolean * link_to_dir,
+                      gboolean * stale_link);
 
 /* Sorting functions */
 int unsorted (file_entry_t * a, file_entry_t * b);
