@@ -10,7 +10,7 @@
    Janne Kukonlehto added much error recovery to them for being used
    in an interactive program.
 
-   Copyright (C) 1994-2020
+   Copyright (C) 1994-2022
    Free Software Foundation, Inc.
 
    Written by:
@@ -492,6 +492,7 @@ overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
 
     vfs_path_t *p;
     char *s1;
+    const char *cs1;
     char s2[BUF_SMALL];
     int w, bw1, bw2;
     unsigned short i;
@@ -522,14 +523,14 @@ overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
     p = vfs_path_from_str (ui->src_filename);
     s1 = vfs_path_to_str_flags (p, 0, VPF_STRIP_HOME | VPF_STRIP_PASSWORD);
     NEW_LABEL (1, s1);
-    vfs_path_free (p);
+    vfs_path_free (p, TRUE);
     g_free (s1);
     /* new file size */
     size_trunc_len (s2, sizeof (s2), ui->src_stat->st_size, 0, panels_options.kilobyte_si);
     NEW_LABEL (2, s2);
     /* new file modification date & time */
-    s1 = (char *) file_date (ui->src_stat->st_mtime);
-    NEW_LABEL (3, s1);
+    cs1 = file_date (ui->src_stat->st_mtime);
+    NEW_LABEL (3, cs1);
 
     /* existing file */
     NEW_LABEL (4, dlg_widgets[4].text);
@@ -537,14 +538,14 @@ overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
     p = vfs_path_from_str (ui->tgt_filename);
     s1 = vfs_path_to_str_flags (p, 0, VPF_STRIP_HOME | VPF_STRIP_PASSWORD);
     NEW_LABEL (5, s1);
-    vfs_path_free (p);
+    vfs_path_free (p, TRUE);
     g_free (s1);
     /* existing file size */
     size_trunc_len (s2, sizeof (s2), ui->dst_stat->st_size, 0, panels_options.kilobyte_si);
     NEW_LABEL (6, s2);
     /* existing file modification date & time */
-    s1 = (char *) file_date (ui->dst_stat->st_mtime);
-    NEW_LABEL (7, s1);
+    cs1 = file_date (ui->dst_stat->st_mtime);
+    NEW_LABEL (7, cs1);
 
     /* will "Append" and "Reget" buttons be in the dialog? */
     do_append = !S_ISDIR (ui->dst_stat->st_mode);
@@ -670,7 +671,7 @@ overwrite_query_dialog (file_op_context_t * ctx, enum OperationMode mode)
     if (result != B_CANCEL)
         ui->dont_overwrite_with_zero = CHECK (dlg_widgets[14].widget)->state;
 
-    dlg_destroy (ui->replace_dlg);
+    widget_destroy (wd);
 
     return (result == B_CANCEL) ? REPLACE_ABORT : (replace_action_t) result;
 
@@ -949,7 +950,7 @@ file_op_context_destroy_ui (file_op_context_t * ctx)
         file_op_context_ui_t *ui = (file_op_context_ui_t *) ctx->ui;
 
         dlg_run_done (ui->op_dlg);
-        dlg_destroy (ui->op_dlg);
+        widget_destroy (WIDGET (ui->op_dlg));
         MC_PTR_FREE (ctx->ui);
     }
 }
@@ -1056,11 +1057,11 @@ file_progress_show_total (file_op_total_context_t * tctx, file_op_context_t * ct
 
     if (ui->time_label != NULL)
     {
-        struct timeval tv_current;
+        gint64 tv_current;
         char buffer4[BUF_TINY];
 
-        gettimeofday (&tv_current, NULL);
-        file_frmt_time (buffer2, tv_current.tv_sec - tctx->transfer_start.tv_sec);
+        tv_current = g_get_monotonic_time ();
+        file_frmt_time (buffer2, (tv_current - tctx->transfer_start) / G_USEC_PER_SEC);
 
         if (ctx->progress_totals_computed)
         {
@@ -1269,15 +1270,15 @@ file_progress_real_query_replace (file_op_context_t * ctx, enum OperationMode mo
 /* --------------------------------------------------------------------------------------------- */
 
 char *
-file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
-                  gboolean only_one,
+file_mask_dialog (file_op_context_t * ctx, FileOperation operation, gboolean only_one,
                   const char *format, const void *text, const char *def_text, gboolean * do_bg)
 {
     size_t fmd_xlen;
     vfs_path_t *vpath;
     gboolean source_easy_patterns = easy_patterns;
     char fmd_buf[BUF_MEDIUM];
-    char *dest_dir, *tmp;
+    char *dest_dir = NULL;
+    char *tmp;
     char *def_text_secure;
 
     if (ctx == NULL)
@@ -1291,7 +1292,7 @@ file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
     /* filter out a possible password from def_text */
     vpath = vfs_path_from_str_flags (def_text, only_one ? VPF_NO_CANON : VPF_NONE);
     tmp = vfs_path_to_str_flags (vpath, 0, VPF_STRIP_PASSWORD);
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
 
     if (source_easy_patterns)
         def_text_secure = strutils_glob_escape (tmp);
@@ -1329,22 +1330,23 @@ file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
     }
 
     {
-        char *source_mask, *orig_mask;
+        char *source_mask = NULL;
+        char *orig_mask;
         int val;
         struct stat buf;
 
         quick_widget_t quick_widgets[] = {
             /* *INDENT-OFF* */
-            QUICK_LABELED_INPUT (fmd_buf, input_label_above,
-                                 easy_patterns ? "*" : "^(.*)$", "input-def", &source_mask,
-                                 NULL, FALSE, FALSE, INPUT_COMPLETE_FILENAMES),
+            QUICK_LABELED_INPUT (fmd_buf, input_label_above, easy_patterns ? "*" : "^(.*)$",
+                                 "input-def", &source_mask, NULL, FALSE, FALSE,
+                                 INPUT_COMPLETE_FILENAMES),
             QUICK_START_COLUMNS,
                 QUICK_SEPARATOR (FALSE),
             QUICK_NEXT_COLUMN,
                 QUICK_CHECKBOX (N_("&Using shell patterns"), &source_easy_patterns, NULL),
             QUICK_STOP_COLUMNS,
-            QUICK_LABELED_INPUT (N_("to:"), input_label_above,
-                                 def_text_secure, "input2", &dest_dir, NULL, FALSE, FALSE, INPUT_COMPLETE_FILENAMES),
+            QUICK_LABELED_INPUT (N_("to:"), input_label_above, def_text_secure, "input2", &dest_dir,
+                                 NULL, FALSE, FALSE, INPUT_COMPLETE_FILENAMES),
             QUICK_SEPARATOR (TRUE),
             QUICK_START_COLUMNS,
                 QUICK_CHECKBOX (N_("Follow &links"), &ctx->follow_links, NULL),
@@ -1369,49 +1371,49 @@ file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
             quick_widgets, NULL, NULL
         };
 
-      ask_file_mask:
-        val = quick_dialog_skip (&qdlg, 4);
-
-        if (val == B_CANCEL)
+        while (TRUE)
         {
-            g_free (def_text_secure);
-            return NULL;
-        }
+            val = quick_dialog_skip (&qdlg, 4);
 
-        ctx->stat_func = ctx->follow_links ? mc_stat : mc_lstat;
+            if (val == B_CANCEL)
+            {
+                g_free (def_text_secure);
+                return NULL;
+            }
 
-        if (ctx->op_preserve)
-        {
-            ctx->preserve = TRUE;
-            ctx->umask_kill = 0777777;
-            ctx->preserve_uidgid = (geteuid () == 0);
-        }
-        else
-        {
-            mode_t i2;
+            ctx->stat_func = ctx->follow_links ? mc_stat : mc_lstat;
 
-            ctx->preserve = ctx->preserve_uidgid = FALSE;
-            i2 = umask (0);
-            umask (i2);
-            ctx->umask_kill = i2 ^ 0777777;
-        }
+            if (ctx->op_preserve)
+            {
+                ctx->preserve = TRUE;
+                ctx->umask_kill = 0777777;
+                ctx->preserve_uidgid = (geteuid () == 0);
+            }
+            else
+            {
+                mode_t i2;
 
-        if ((dest_dir == NULL) || (*dest_dir == '\0'))
-        {
-            g_free (def_text_secure);
-            g_free (source_mask);
-            g_free (dest_dir);
-            return NULL;
-        }
+                ctx->preserve = ctx->preserve_uidgid = FALSE;
+                i2 = umask (0);
+                umask (i2);
+                ctx->umask_kill = i2 ^ 0777777;
+            }
 
-        ctx->search_handle = mc_search_new (source_mask, NULL);
+            if (dest_dir == NULL || *dest_dir == '\0')
+            {
+                g_free (def_text_secure);
+                g_free (source_mask);
+                g_free (dest_dir);
+                return NULL;
+            }
+
+            ctx->search_handle = mc_search_new (source_mask, NULL);
+            if (ctx->search_handle != NULL)
+                break;
 
-        if (ctx->search_handle == NULL)
-        {
             message (D_ERROR, MSG_ERROR, _("Invalid source pattern '%s'"), source_mask);
-            g_free (dest_dir);
-            g_free (source_mask);
-            goto ask_file_mask;
+            MC_PTR_FREE (dest_dir);
+            MC_PTR_FREE (source_mask);
         }
 
         g_free (def_text_secure);
@@ -1456,7 +1458,7 @@ file_mask_dialog (file_op_context_t * ctx, FileOperation operation,
             dest_dir = g_strdup ("./");
         }
 
-        vfs_path_free (vpath);
+        vfs_path_free (vpath, TRUE);
 
         if (val == B_USER)
             *do_bg = TRUE;
