@@ -1,7 +1,7 @@
 /*
    Main dialog (file panels) of the Midnight Commander
 
-   Copyright (C) 1994-2020
+   Copyright (C) 1994-2022
    Free Software Foundation, Inc.
 
    Written by:
@@ -61,7 +61,7 @@
 #include "src/execute.h"        /* toggle_subshell */
 #include "src/setup.h"          /* variables */
 #include "src/learn.h"          /* learn_keys() */
-#include "src/keybind-defaults.h"
+#include "src/keymap.h"
 #include "lib/fileloc.h"        /* MC_FILEPOS_FILE */
 #include "lib/keybind.h"
 #include "lib/event.h"
@@ -135,7 +135,7 @@ static menu_t *left_menu, *right_menu;
 static void
 stop_dialogs (void)
 {
-    dlg_stop (midnight_dlg);
+    dlg_stop (filemanager);
 
     if (top_dlg != NULL)
         dlg_stop (DIALOG (top_dlg->data));
@@ -148,14 +148,14 @@ treebox_cmd (void)
 {
     char *sel_dir;
 
-    sel_dir = tree_box (selection (current_panel)->fname);
+    sel_dir = tree_box (selection (current_panel)->fname->str);
     if (sel_dir != NULL)
     {
         vfs_path_t *sel_vdir;
 
         sel_vdir = vfs_path_from_str (sel_dir);
         panel_cd (current_panel, sel_vdir, cd_exact);
-        vfs_path_free (sel_vdir);
+        vfs_path_free (sel_vdir, TRUE);
         g_free (sel_dir);
     }
 }
@@ -199,7 +199,7 @@ create_panel_menu (void)
     entries =
         g_list_prepend (entries,
                         menu_entry_create (_("&Listing format..."), CK_SetupListingFormat));
-    entries = g_list_prepend (entries, menu_entry_create (_("&Sort order..."), CK_Sort));
+    entries = g_list_prepend (entries, menu_entry_create (_("S&ort order..."), CK_Sort));
     entries = g_list_prepend (entries, menu_entry_create (_("&Filter..."), CK_Filter));
 #ifdef HAVE_CHARSET
     entries = g_list_prepend (entries, menu_entry_create (_("&Encoding..."), CK_SelectCodepage));
@@ -212,10 +212,7 @@ create_panel_menu (void)
     entries = g_list_prepend (entries, menu_entry_create (_("S&hell link..."), CK_ConnectFish));
 #endif
 #ifdef ENABLE_VFS_SFTP
-    entries = g_list_prepend (entries, menu_entry_create (_("S&FTP link..."), CK_ConnectSftp));
-#endif
-#ifdef ENABLE_VFS_SMB
-    entries = g_list_prepend (entries, menu_entry_create (_("SM&B link..."), CK_ConnectSmb));
+    entries = g_list_prepend (entries, menu_entry_create (_("&SFTP link..."), CK_ConnectSftp));
 #endif
     entries = g_list_prepend (entries, menu_entry_create (_("Paneli&ze"), CK_Panelize));
     entries = g_list_prepend (entries, menu_separator_create ());
@@ -663,7 +660,7 @@ create_panels (void)
         else
             vpath = vfs_path_append_new (original_dir, other_dir, (char *) NULL);
         mc_chdir (vpath);
-        vfs_path_free (vpath);
+        vfs_path_free (vpath, TRUE);
     }
     create_panel (other_index, other_mode);
 
@@ -679,7 +676,7 @@ create_panels (void)
         else
             vpath = vfs_path_append_new (original_dir, current_dir, (char *) NULL);
         mc_chdir (vpath);
-        vfs_path_free (vpath);
+        vfs_path_free (vpath, TRUE);
     }
     create_panel (current_index, current_mode);
 
@@ -690,7 +687,7 @@ create_panels (void)
     else
         current_panel = left_panel;
 
-    vfs_path_free (original_dir);
+    vfs_path_free (original_dir, TRUE);
 
 #ifdef ENABLE_VFS
     mc_event_add (MCEVENT_GROUP_CORE, "vfs_timestamp", check_other_panel_timestamp, NULL, NULL);
@@ -724,7 +721,7 @@ midnight_put_panel_path (WPanel * panel)
     if (!IS_PATH_SEP (cwd_vpath_str[strlen (cwd_vpath_str) - 1]))
         command_insert (cmdline, PATH_SEP_STR, FALSE);
 
-    vfs_path_free (cwd_vpath);
+    vfs_path_free (cwd_vpath, TRUE);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -740,9 +737,10 @@ put_link (WPanel * panel)
         vfs_path_t *vpath;
         int i;
 
-        vpath = vfs_path_append_new (panel->cwd_vpath, selection (panel)->fname, (char *) NULL);
+        vpath =
+            vfs_path_append_new (panel->cwd_vpath, selection (panel)->fname->str, (char *) NULL);
         i = mc_readlink (vpath, buffer, sizeof (buffer) - 1);
-        vfs_path_free (vpath);
+        vfs_path_free (vpath, TRUE);
 
         if (i > 0)
         {
@@ -790,7 +788,7 @@ put_current_selected (void)
         tmp = vfs_path_as_str (selected_name);
     }
     else
-        tmp = selection (current_panel)->fname;
+        tmp = selection (current_panel)->fname->str;
 
     command_insert (cmdline, tmp, TRUE);
 }
@@ -810,12 +808,12 @@ put_tagged (WPanel * panel)
         for (i = 0; i < panel->dir.len; i++)
         {
             if (panel->dir.list[i].f.marked)
-                command_insert (cmdline, panel->dir.list[i].fname, TRUE);
+                command_insert (cmdline, panel->dir.list[i].fname->str, TRUE);
         }
     }
     else
     {
-        command_insert (cmdline, panel->dir.list[panel->selected].fname, TRUE);
+        command_insert (cmdline, panel->dir.list[panel->selected].fname->str, TRUE);
     }
     input_enable_update (cmdline);
 }
@@ -858,11 +856,6 @@ setup_mc (void)
 #endif /* HAVE_CHARSET */
 #endif /* HAVE_SLANG */
 
-#ifdef ENABLE_SUBSHELL
-    if (mc_global.tty.use_subshell)
-        add_select_channel (mc_global.tty.subshell_pty, load_prompt, NULL);
-#endif /* !ENABLE_SUBSHELL */
-
     if ((tty_baudrate () < 9600) || mc_global.tty.slow_terminal)
         verbose = FALSE;
 }
@@ -881,7 +874,7 @@ setup_dummy_mc (void)
     vpath = vfs_path_from_str (d);
     ret = mc_chdir (vpath);
     (void) ret;
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
     g_free (d);
 }
 
@@ -906,18 +899,18 @@ done_mc (void)
 static void
 create_file_manager (void)
 {
-    Widget *w = WIDGET (midnight_dlg);
-    WGroup *g = GROUP (midnight_dlg);
+    Widget *w = WIDGET (filemanager);
+    WGroup *g = GROUP (filemanager);
 
     w->keymap = filemanager_map;
     w->ext_keymap = filemanager_x_map;
 
-    midnight_dlg->get_shortcut = midnight_get_shortcut;
-    midnight_dlg->get_title = midnight_get_title;
+    filemanager->get_shortcut = midnight_get_shortcut;
+    filemanager->get_title = midnight_get_title;
     /* allow rebind tab */
     widget_want_tab (w, TRUE);
 
-    the_menubar = menubar_new (NULL, menubar_visible);
+    the_menubar = menubar_new (NULL);
     group_add_widget (g, the_menubar);
     init_menu ();
 
@@ -938,9 +931,16 @@ create_file_manager (void)
     the_prompt->transparent = TRUE;
     group_add_widget (g, the_prompt);
 
-    the_bar = buttonbar_new (mc_global.keybar_visible);
+    the_bar = buttonbar_new ();
     group_add_widget (g, the_bar);
     midnight_set_buttonbar (the_bar);
+
+#ifdef ENABLE_SUBSHELL
+    /* Must be done after creation of cmdline and promt widgets to avoid potential
+       NULL dereference in load_prompt() -> ... -> setup_cmdline() -> label_set_text(). */
+    if (mc_global.tty.use_subshell)
+        add_select_channel (mc_global.tty.subshell_pty, load_prompt, NULL);
+#endif /* !ENABLE_SUBSHELL */
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -955,7 +955,7 @@ prepend_cwd_on_local (const char *filename)
     if (!vfs_file_is_local (vpath) || g_path_is_absolute (filename))
         return vpath;
 
-    vfs_path_free (vpath);
+    vfs_path_free (vpath, TRUE);
 
     return vfs_path_append_new (vfs_get_raw_current_dir (), filename, (char *) NULL);
 }
@@ -985,7 +985,7 @@ mc_maybe_editor_or_viewer (void)
                 vpath = prepend_cwd_on_local ((char *) mc_run_param0);
 
             ret = view_file (vpath, FALSE, TRUE);
-            vfs_path_free (vpath);
+            vfs_path_free (vpath, TRUE);
             break;
         }
 #ifdef USE_DIFF_VIEW
@@ -1008,7 +1008,7 @@ show_editor_viewer_history (void)
     char *s;
     int act;
 
-    s = show_file_history (WIDGET (midnight_dlg), &act);
+    s = show_file_history (WIDGET (filemanager), &act);
     if (s != NULL)
     {
         vfs_path_t *s_vpath;
@@ -1038,7 +1038,7 @@ show_editor_viewer_history (void)
         }
 
         g_free (s);
-        vfs_path_free (s_vpath);
+        vfs_path_free (s_vpath, TRUE);
     }
 }
 
@@ -1264,11 +1264,6 @@ midnight_execute_cmd (Widget * sender, long command)
         sftplink_cmd ();
         break;
 #endif
-#ifdef ENABLE_VFS_SMB
-    case CK_ConnectSmb:
-        smblink_cmd ();
-        break;
-#endif /* ENABLE_VFS_SMB */
     case CK_Panelize:
         cd_panelize_cmd ();
         break;
@@ -1365,7 +1360,7 @@ midnight_execute_cmd (Widget * sender, long command)
     case CK_Select:
     case CK_Unselect:
     case CK_SelectInvert:
-        res = send_message (current_panel, midnight_dlg, MSG_ACTION, command, NULL);
+        res = send_message (current_panel, filemanager, MSG_ACTION, command, NULL);
         break;
     case CK_Shell:
         toggle_subshell ();
@@ -1377,7 +1372,7 @@ midnight_execute_cmd (Widget * sender, long command)
         sort_cmd ();
         break;
     case CK_ExtendedKeyMap:
-        WIDGET (midnight_dlg)->ext_mode = TRUE;
+        WIDGET (filemanager)->ext_mode = TRUE;
         break;
     case CK_Suspend:
         mc_event_raise (MCEVENT_GROUP_CORE, "suspend", NULL);
@@ -1557,14 +1552,13 @@ midnight_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
             {
                 /* Special treatement, since the input line will eat them */
                 if (parm == '+')
-                    return send_message (current_panel, midnight_dlg, MSG_ACTION, CK_Select, NULL);
+                    return send_message (current_panel, filemanager, MSG_ACTION, CK_Select, NULL);
 
                 if (parm == '\\' || parm == '-')
-                    return send_message (current_panel, midnight_dlg, MSG_ACTION, CK_Unselect,
-                                         NULL);
+                    return send_message (current_panel, filemanager, MSG_ACTION, CK_Unselect, NULL);
 
                 if (parm == '*')
-                    return send_message (current_panel, midnight_dlg, MSG_ACTION, CK_SelectInvert,
+                    return send_message (current_panel, filemanager, MSG_ACTION, CK_SelectInvert,
                                          NULL);
             }
             else if (!command_prompt || input_is_empty (cmdline))
@@ -1573,14 +1567,13 @@ midnight_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
                  * first char on input line
                  */
                 if (parm == '+')
-                    return send_message (current_panel, midnight_dlg, MSG_ACTION, CK_Select, NULL);
+                    return send_message (current_panel, filemanager, MSG_ACTION, CK_Select, NULL);
 
                 if (parm == '\\' || parm == '-')
-                    return send_message (current_panel, midnight_dlg, MSG_ACTION, CK_Unselect,
-                                         NULL);
+                    return send_message (current_panel, filemanager, MSG_ACTION, CK_Unselect, NULL);
 
                 if (parm == '*')
-                    return send_message (current_panel, midnight_dlg, MSG_ACTION, CK_SelectInvert,
+                    return send_message (current_panel, filemanager, MSG_ACTION, CK_SelectInvert,
                                          NULL);
             }
         }
@@ -1617,7 +1610,7 @@ midnight_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void
         /* Handle shortcuts, menu, and buttonbar. */
         return midnight_execute_cmd (sender, parm);
 
-    case MSG_END:
+    case MSG_DESTROY:
         panel_deinit ();
         return MSG_HANDLED;
 
@@ -1636,7 +1629,7 @@ update_menu (void)
     menu_set_name (left_menu, panels_layout.horizontal_split ? _("&Above") : _("&Left"));
     menu_set_name (right_menu, panels_layout.horizontal_split ? _("&Below") : _("&Right"));
     menubar_arrange (the_menubar);
-    menubar_set_visible (the_menubar, menubar_visible);
+    widget_set_visibility (WIDGET (the_menubar), menubar_visible);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -1644,7 +1637,7 @@ update_menu (void)
 void
 midnight_set_buttonbar (WButtonBar * b)
 {
-    Widget *w = WIDGET (midnight_dlg);
+    Widget *w = WIDGET (filemanager);
 
     buttonbar_set_label (b, 1, Q_ ("ButtonBar|Help"), w->keymap, NULL);
     buttonbar_set_label (b, 2, Q_ ("ButtonBar|Menu"), w->keymap, NULL);
@@ -1669,7 +1662,7 @@ get_random_hint (gboolean force)
     static const gint64 update_period = 60 * G_USEC_PER_SEC;
     static gint64 tv = 0;
 
-    char *data, *result = NULL, *eop;
+    char *data, *result, *eop;
     size_t len, start;
     GIConv conv;
 
@@ -1708,19 +1701,18 @@ get_random_hint (gboolean force)
     /* hint files are stored in utf-8 */
     /* try convert hint file from utf-8 to terminal encoding */
     conv = str_crt_conv_from ("UTF-8");
-    if (conv != INVALID_CONV)
+    if (conv == INVALID_CONV)
+        result = g_strndup (data + start, len - start);
+    else
     {
         GString *buffer;
+        gboolean nok;
 
         buffer = g_string_sized_new (len - start);
-        if (str_convert (conv, &data[start], buffer) != ESTR_FAILURE)
-            result = g_string_free (buffer, FALSE);
-        else
-            g_string_free (buffer, TRUE);
+        nok = (str_convert (conv, data + start, buffer) == ESTR_FAILURE);
+        result = g_string_free (buffer, nok);
         str_close_conv (conv);
     }
-    else
-        result = g_strndup (data + start, len - start);
 
     g_free (data);
     return result;
@@ -1759,7 +1751,7 @@ load_hint (gboolean force)
     {
         char text[BUF_SMALL];
 
-        g_snprintf (text, sizeof (text), _("GNU Midnight Commander %s\n"), VERSION);
+        g_snprintf (text, sizeof (text), _("GNU Midnight Commander %s\n"), mc_global.mc_version);
         set_hintbar (text);
     }
 }
@@ -1775,7 +1767,7 @@ WPanel *
 change_panel (void)
 {
     input_complete_free (cmdline);
-    group_select_next_widget (GROUP (midnight_dlg));
+    group_select_next_widget (GROUP (filemanager));
     return current_panel;
 }
 
@@ -1816,8 +1808,8 @@ do_nc (void)
     edit_stack_init ();
 #endif
 
-    midnight_dlg = dlg_create (FALSE, 0, 0, 1, 1, WPOS_FULLSCREEN, FALSE, dialog_colors,
-                               midnight_callback, NULL, "[main]", NULL);
+    filemanager = dlg_create (FALSE, 0, 0, 1, 1, WPOS_FULLSCREEN, FALSE, dialog_colors,
+                              midnight_callback, NULL, "[main]", NULL);
 
     /* Check if we were invoked as an editor or file viewer */
     if (mc_global.mc_run_mode != MC_RUN_FULL)
@@ -1828,19 +1820,19 @@ do_nc (void)
     else
     {
         /* We only need the first idle event to show user menu after start */
-        widget_idle (WIDGET (midnight_dlg), TRUE);
+        widget_idle (WIDGET (filemanager), TRUE);
 
         setup_mc ();
         mc_filehighlight = mc_fhl_new (TRUE);
 
         create_file_manager ();
-        (void) dlg_run (midnight_dlg);
+        (void) dlg_run (filemanager);
 
         mc_fhl_free (&mc_filehighlight);
 
         ret = TRUE;
 
-        /* dlg_destroy destroys even current_panel->cwd_vpath, so we have to save a copy :) */
+        /* widget_destroy destroys even current_panel->cwd_vpath, so we have to save a copy :) */
         if (mc_args__last_wd_file != NULL && vfs_current_is_local ())
             last_wd_string = g_strdup (vfs_path_as_str (current_panel->cwd_vpath));
 
@@ -1854,7 +1846,7 @@ do_nc (void)
     mc_global.midnight_shutdown = TRUE;
     dialog_switch_shutdown ();
     done_mc ();
-    dlg_destroy (midnight_dlg);
+    widget_destroy (WIDGET (filemanager));
     current_panel = NULL;
 
 #ifdef USE_INTERNAL_EDIT
