@@ -2,7 +2,7 @@
    Internal file viewer for the Midnight Commander
    Function for paint dialogs
 
-   Copyright (C) 1994-2020
+   Copyright (C) 1994-2022
    Free Software Foundation, Inc.
 
    Written by:
@@ -52,14 +52,6 @@
 
 /*** global variables ****************************************************************************/
 
-mcview_search_options_t mcview_search_options = {
-    .type = MC_SEARCH_T_NORMAL,
-    .case_sens = FALSE,
-    .backwards = FALSE,
-    .whole_words = FALSE,
-    .all_codepages = FALSE
-};
-
 /*** file scope macro definitions ****************************************************************/
 
 /*** file scope type declarations ****************************************************************/
@@ -133,30 +125,10 @@ mcview_dialog_search (WView * view)
     }
 #endif
 
-    g_free (view->last_search_string);
+    mcview_search_deinit (view);
     view->last_search_string = exp;
-    mcview_nroff_seq_free (&view->search_nroff_seq);
-    mc_search_free (view->search);
 
-#ifdef HAVE_CHARSET
-    view->search = mc_search_new (view->last_search_string, cp_source);
-#else
-    view->search = mc_search_new (view->last_search_string, NULL);
-#endif
-    view->search_nroff_seq = mcview_nroff_seq_new (view);
-    if (view->search != NULL)
-    {
-        view->search->search_type = mcview_search_options.type;
-#ifdef HAVE_CHARSET
-        view->search->is_all_charsets = mcview_search_options.all_codepages;
-#endif
-        view->search->is_case_sensitive = mcview_search_options.case_sens;
-        view->search->whole_words = mcview_search_options.whole_words;
-        view->search->search_fn = mcview_search_cmd_callback;
-        view->search->update_fn = mcview_search_update_cmd_callback;
-    }
-
-    return (view->search != NULL);
+    return mcview_search_init (view);
 }
 
 /* --------------------------------------------------------------------------------------------- */
