@@ -1,7 +1,7 @@
 /*
    lib/strutil - tests for lib/strutil/parse_integer function.
 
-   Copyright (C) 2013-2020
+   Copyright (C) 2013-2022
    Free Software Foundation, Inc.
 
    Written by:
@@ -130,9 +130,9 @@ START_PARAMETRIZED_TEST (parse_integer_test, parse_integer_test_ds)
     actual_result = parse_integer (data->haystack, &invalid);
 
     /* then */
-    fail_unless (invalid == data->invalid && actual_result == data->expected_result,
-                 "actial ( %" PRIuMAX ") not equal to\nexpected (%" PRIuMAX ")",
-                 actual_result, data->expected_result);
+    ck_assert_msg (invalid == data->invalid && actual_result == data->expected_result,
+                   "actial ( %" PRIuMAX ") not equal to\nexpected (%" PRIuMAX ")",
+                   actual_result, data->expected_result);
 }
 /* *INDENT-OFF* */
 END_PARAMETRIZED_TEST
@@ -143,11 +143,9 @@ END_PARAMETRIZED_TEST
 int
 main (void)
 {
-    int number_failed;
+    TCase *tc_core;
 
-    Suite *s = suite_create (TEST_SUITE_NAME);
-    TCase *tc_core = tcase_create ("Core");
-    SRunner *sr;
+    tc_core = tcase_create ("Core");
 
     tcase_add_checked_fixture (tc_core, setup, teardown);
 
@@ -155,13 +153,7 @@ main (void)
     mctest_add_parameterized_test (tc_core, parse_integer_test, parse_integer_test_ds);
     /* *********************************** */
 
-    suite_add_tcase (s, tc_core);
-    sr = srunner_create (s);
-    srunner_set_log (sr, "parse_integer.log");
-    srunner_run_all (sr, CK_ENV);
-    number_failed = srunner_ntests_failed (sr);
-    srunner_free (sr);
-    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
+    return mctest_run_all (tc_core);
 }
 
 /* --------------------------------------------------------------------------------------------- */
